from __future__ import annotations

from dataclasses import asdict
from typing import Any

from ozi_spec import METADATA
from prompt_toolkit.shortcuts import button_dialog  # pyright: ignore
from prompt_toolkit.shortcuts import checkboxlist_dialog  # pyright: ignore
from prompt_toolkit.shortcuts import radiolist_dialog  # pyright: ignore
from prompt_toolkit.shortcuts import yes_no_dialog  # pyright: ignore

from ozi_core._i18n import TRANSLATION
from ozi_core.config import OziConfig
from ozi_core.config import read_user_config
from ozi_core.config import write_user_config
from ozi_core.trove import Prefix
from ozi_core.trove import from_prefix
from ozi_core.ui._style import _style
from ozi_core.ui.dialog import admonition_dialog
from ozi_core.ui.dialog import input_dialog
from ozi_core.ui.menu import MenuButton
from ozi_core.ui.menu import checkbox


def main_menu(  # pragma: no cover
    project: Any,
    output: dict[str, list[str]],
    prefix: dict[str, str],
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]:
    while True:
        match button_dialog(
            title=TRANSLATION('new-dlg-title'),
            text=TRANSLATION('main-menu-text'),
            buttons=[
                MenuButton.METADATA._tuple,
                MenuButton.OPTIONS._tuple,
                MenuButton.RESET._tuple,
                MenuButton.EXIT._tuple,
                MenuButton.EDIT._tuple,
                MenuButton.BACK._tuple,
            ],
            style=_style,
        ).run():
            case MenuButton.BACK.value:
                break
            case MenuButton.RESET.value:
                if yes_no_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    text=TRANSLATION('main-menu-yn-reset'),
                    style=_style,
                    yes_text=MenuButton.YES._str,
                    no_text=MenuButton.NO._str,
                ).run():
                    return ['interactive', '.'], output, prefix
            case MenuButton.EXIT.value:
                if yes_no_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    text=TRANSLATION('main-menu-yn-exit'),
                    style=_style,
                    yes_text=MenuButton.YES._str,
                    no_text=MenuButton.NO._str,
                ).run():
                    return [], output, prefix
            case MenuButton.EDIT.value:
                result, output, prefix = edit_menu(project, output, prefix)
                if isinstance(result, list):
                    return result, output, prefix
            case MenuButton.OPTIONS.value:
                result, output, prefix = options_menu(project, output, prefix)
                if isinstance(result, list):
                    return result, output, prefix
            case MenuButton.METADATA.value:
                if admonition_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    heading_label=TRANSLATION('adm-metadata'),
                    text='\n'.join(
                        prefix.values() if len(prefix) > 0 else {'Name:': 'Name:'},
                    ),
                    ok_text=MenuButton.PROMPT._str,
                    cancel_text=MenuButton.BACK._str,
                ).run():
                    break
    return None, output, prefix


def options_menu(  # pragma: no cover
    project: Any,
    output: dict[str, list[str]],
    prefix: dict[str, str],
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]:
    _default: str | list[str] | None = None
    while True:
        match radiolist_dialog(
            title=TRANSLATION('new-dlg-title'),
            text=TRANSLATION('opt-menu-title'),
            values=[
                (
                    'enable_cython',
                    TRANSLATION(
                        'opt-menu-enable-cython',
                        value=checkbox(project.enable_cython),
                    ),
                ),
                (
                    'enable_uv',
                    TRANSLATION(
                        'opt-menu-enable-uv',
                        value=checkbox(project.enable_uv),
                    ),
                ),
                (
                    'github_harden_runner',
                    TRANSLATION(
                        'opt-menu-github-harden-runner',
                        value=checkbox(project.github_harden_runner),
                    ),
                ),
                (
                    'strict',
                    TRANSLATION('opt-menu-strict', value=checkbox(project.strict)),
                ),
                (
                    'update_wrapfile',
                    TRANSLATION(
                        'opt-menu-update-wrapfile', value=checkbox(project.update_wrapfile)
                    ),
                ),
                (
                    'verify_email',
                    TRANSLATION(
                        'opt-menu-verify-email',
                        value=checkbox(project.verify_email),
                    ),
                ),
                ('allow_file', TRANSLATION('opt-menu-allow-file')),
                ('ci_provider', TRANSLATION('opt-menu-ci-provider')),
                ('copyright_head', TRANSLATION('opt-menu-copyright-head')),
                (
                    'language',
                    TRANSLATION(
                        'opt-menu-language',
                        value=TRANSLATION(f'lang-{TRANSLATION.locale}'),
                    ),
                ),
            ],
            style=_style,
            cancel_text=MenuButton.BACK._str,
            ok_text=MenuButton.OK._str,
        ).run():
            case x if x and x in (
                'enable_cython',
                'enable_uv',
                'github_harden_runner',
                'verify_email',
                'update_wrapfile',
            ):
                for i in (
                    f'--{x.replace("_", "-")}',
                    f'--no-{x.replace("_", "-")}',
                ):
                    if i in output:
                        output.pop(i)
                setting = getattr(project, x)
                if setting is None:
                    setattr(project, x, True)
                else:
                    flag = '' if not setting else 'no-'
                    output.update(
                        {
                            f'--{flag}{x.replace("_", "-")}': [
                                f'--{flag}{x.replace("_", "-")}',
                            ],
                        },
                    )
                    setattr(project, x, not setting)
            case x if x and x == 'strict':
                for i in ('--strict', '--no-strict'):
                    if i in output:
                        output.pop(i)
                setting = getattr(project, x)
                if setting is None:
                    setattr(project, x, False)
                else:
                    flag = '' if setting else 'no-'
                    output.update(
                        {
                            f'--{flag}{x.replace("_", "-")}': [
                                f'--{flag}{x.replace("_", "-")}',
                            ],
                        },
                    )
                    setattr(project, x, not setting)
            case x if x and x == 'copyright_head':
                _default = output.setdefault(
                    '--copyright-head',
                    [
                        'Part of {project_name}.\nSee LICENSE.txt in the project root for details.',  # noqa: B950, RUF100, E501
                    ],
                )
                result = input_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    text=TRANSLATION('opt-menu-copyright-head-input'),
                    style=_style,
                    cancel_text=MenuButton.BACK._str,
                    ok_text=MenuButton.OK._str,
                    default=_default[0],
                    multiline=True,
                ).run()
                if result in _default:
                    project.copyright_head = result
                    output.update({'--copyright-head': [project.copyright_head]})
            case x if x and x == 'allow_file':
                _default = output.setdefault(
                    '--allow-file',
                    list(METADATA.spec.python.src.allow_files),
                )
                result = input_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    text=TRANSLATION('opt-menu-allow-file-input'),
                    style=_style,
                    cancel_text=MenuButton.BACK._str,
                    ok_text=MenuButton.OK._str,
                    default=','.join(_default),
                ).run()
                if result != ','.join(_default) and result is not None:
                    project.allow_file = [i.strip() for i in result.split(',')]
                    output.update({'--allow-file': [result]})
            case x if x and x == 'ci_provider':
                _default = output.setdefault('--ci-provider', ['github'])
                result = radiolist_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    text=TRANSLATION('opt-menu-ci-provider-input'),
                    values=[('github', 'GitHub')],
                    cancel_text=MenuButton.BACK._str,
                    ok_text=MenuButton.OK._str,
                    default=_default[0],
                    style=_style,
                ).run()
                if result in _default and result is not None:
                    project.ci_provider = result
                    output.update({'--ci-provider': [project.ci_provider]})
            case x if x == 'language':
                result = radiolist_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    text=TRANSLATION('opt-menu-language-text'),
                    values=list(
                        zip(
                            TRANSLATION.data.keys(),
                            [TRANSLATION(f'lang-{i}') for i in TRANSLATION.data.keys()],
                        ),
                    ),
                    cancel_text=MenuButton.BACK._str,
                    ok_text=MenuButton.OK._str,
                    default=TRANSLATION.locale,
                    style=_style,
                ).run()
                if result is not None:
                    TRANSLATION.locale = result
            case _:
                if yes_no_dialog(
                    title=TRANSLATION('new-dlg-title'),
                    text=TRANSLATION('opt-menu-save-config'),
                    yes_text=MenuButton.YES._str,
                    no_text=MenuButton.NO._str,
                    style=_style,
                ).run():
                    config = asdict(read_user_config())
                    config['new'].update(**vars(project))
                    config['interactive'].update(language=TRANSLATION.locale)
                    write_user_config(OziConfig(**config))
                break
    return None, output, prefix


def edit_menu(  # pragma: no cover
    project: Any,
    output: dict[str, list[str]],
    prefix: dict[str, str],
) -> tuple[None | list[str] | bool, dict[str, list[str]], dict[str, str]]:
    while True:
        match radiolist_dialog(
            title=TRANSLATION('new-dlg-title'),
            text=TRANSLATION('edit-menu-text'),
            values=[
                ('name', TRANSLATION('edit-menu-btn-name')),
                ('summary', TRANSLATION('edit-menu-btn-summary')),
                ('keywords', TRANSLATION('edit-menu-btn-keywords')),
                ('home_page', TRANSLATION('edit-menu-btn-homepage')),
                ('author', TRANSLATION('edit-menu-btn-author')),
                ('author_email', TRANSLATION('edit-menu-btn-email')),
                ('license_', TRANSLATION('edit-menu-btn-license')),
                (
                    'license_expression',
                    TRANSLATION('edit-menu-btn-license-expression'),
                ),
                ('license_file', TRANSLATION('edit-menu-btn-license-file')),
                ('maintainer', TRANSLATION('edit-menu-btn-maintainer')),
                (
                    'maintainer_email',
                    TRANSLATION('edit-menu-btn-maintainer-email'),
                ),
                ('project_urls', TRANSLATION('edit-menu-btn-project-url')),
                (
                    'requires_dist',
                    TRANSLATION('edit-menu-btn-requires-dist'),
                ),
                ('audience', TRANSLATION('edit-menu-btn-audience')),
                ('environment', TRANSLATION('edit-menu-btn-environment')),
                ('framework', TRANSLATION('edit-menu-btn-framework')),
                ('language', TRANSLATION('edit-menu-btn-language')),
                ('status', TRANSLATION('edit-menu-btn-status')),
                ('topic', TRANSLATION('edit-menu-btn-topic')),
                ('typing', TRANSLATION('edit-menu-btn-typing')),
                ('readme_type', TRANSLATION('edit-menu-btn-readme-type')),
            ],
            cancel_text=MenuButton.BACK._str,
            ok_text=MenuButton.OK._str,
            style=_style,
        ).run():
            case None:
                return None, output, prefix
            case x if x and isinstance(x, str):
                project_name = prefix.get('Name', '').replace('Name', '').strip(': ')
                match x:
                    case x if x == 'name':
                        result, output, prefix = project.name(output, prefix)
                        if isinstance(result, list):
                            return result, output, prefix
                    case x if x == 'license_expression':
                        result, output, prefix = project.license_expression(
                            project_name,
                            prefix.get(
                                'License',
                                '',
                            )
                            .replace(
                                'License',
                                '',
                            )
                            .strip(': '),
                            output,
                            prefix,
                        )
                        if isinstance(result, list):
                            return result, output, prefix
                    case x if x == 'license_':
                        result, output, prefix = project.license_(
                            project_name,
                            output,
                            prefix,
                        )
                        if isinstance(result, str):
                            result, output, prefix = project.license_expression(
                                project_name,
                                result,
                                output,
                                prefix,
                            )
                        if isinstance(result, list):  # pyright: ignore
                            return result, output, prefix
                    case x if x and x in (
                        'audience',
                        'environment',
                        'framework',
                        'language',
                        'status',
                        'topic',
                    ):
                        output.setdefault(f'--{x}', [])
                        config = asdict(read_user_config())
                        header = getattr(Prefix(), x)
                        classifier = checkboxlist_dialog(
                            values=sorted(
                                (
                                    zip(
                                        from_prefix(header),
                                        from_prefix(header),
                                    )
                                ),
                            ),
                            title=TRANSLATION('new-dlg-title'),
                            text=TRANSLATION(
                                'pro-classifier-cbl',
                                key=TRANSLATION(f'edit-menu-btn-{x}'),
                            ),
                            default_values=(
                                config['new']['language'] if x == 'language' else None
                            ),
                            style=_style,
                            ok_text=MenuButton.OK._str,
                            cancel_text=MenuButton.BACK._str,
                        ).run()
                        if classifier is not None:
                            for i in classifier:
                                output[f'--{x}'].append(i)
                                if x in ['language']:
                                    config['new'].update(
                                        {x: output[f'--{x}']} | vars(project)
                                    )
                                    write_user_config(OziConfig(**config))
                        prefix.update(
                            (
                                {
                                    f'{header}': f'{header}{classifier}',
                                }
                                if classifier
                                else {}
                            ),
                        )
                    case x if x and x in (
                        'author',
                        'author_email',
                        'maintainer',
                        'maintainer_email',
                        'readme_type',
                    ):
                        result, output, prefix = getattr(project, x)(
                            project_name,
                            output,
                            prefix,
                        )
                        if isinstance(result, list):
                            return result, output, prefix
                        if yes_no_dialog(
                            title=TRANSLATION('new-dlg-title'),
                            text=TRANSLATION('opt-menu-save-config'),
                            yes_text=MenuButton.YES._str,
                            no_text=MenuButton.NO._str,
                            style=_style,
                        ).run():
                            config = asdict(read_user_config())
                            config['new'].update(
                                {x: output[f'--{x.replace("_", "-")}'][0]} | vars(project)
                            )
                            write_user_config(OziConfig(**config))
                    case x:
                        result, output, prefix = getattr(project, x)(
                            project_name,
                            output,
                            prefix,
                        )
                        if isinstance(result, list):
                            return result, output, prefix



"""
This type stub file was generated by pyright.
"""

from typing import TYPE_CHECKING

from prompt_toolkit.validation import Validator

if TYPE_CHECKING:
    ...

class Project:
    def __init__(
        self,
        allow_file: list[str] | None = ...,
        ci_provider: str | None = ...,
        copyright_head: str | None = ...,
        enable_cython: bool | None = ...,
        enable_uv: bool | None = ...,
        github_harden_runner: bool | None = ...,
        strict: bool | None = ...,
        verify_email: bool | None = ...,
        check_package_exists: bool | None = ...,
    ) -> None: ...
    def __call__(self) -> list[str]: ...
    def name(
        self, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def summary(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def keywords(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def license_file(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def home_page(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def author(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def author_email(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def license_(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str, dict[str, list[str]], dict[str, str]]: ...
    def license_expression(
        self,
        project_name: str,
        _license: str,
        output: dict[str, list[str]],
        prefix: dict[str, str],
    ) -> tuple[None | list[str] | str, dict[str, list[str]], dict[str, str]]: ...
    def maintainer(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def maintainer_email(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[None | list[str] | str | bool, dict[str, list[str]], dict[str, str]]: ...
    def requires_dist(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[list[str] | str | bool | None, dict[str, list[str]], dict[str, str]]: ...
    def readme_type(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[str | list[str], dict[str, list[str]], dict[str, str]]: ...
    def typing(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[str | list[str], dict[str, list[str]], dict[str, str]]: ...
    def project_urls(
        self, project_name: str, output: dict[str, list[str]], prefix: dict[str, str]
    ) -> tuple[str, dict[str, list[str]], dict[str, str]]: ...
    def header_input(
        self,
        label: str,
        output: dict[str, list[str]],
        prefix: dict[str, str],
        *args: str,
        validator: Validator | None = ...,
        split_on: str | None = ...
    ) -> tuple[bool | None | list[str], dict[str, list[str]], dict[str, str]]: ...



"""
This type stub file was generated by pyright.
"""

from pathlib import Path
from typing import Generator

"""Build definition check utilities."""
IGNORE_MISSING = ...

def unroll_subdirs(target: Path, rel_path: Path) -> str:
    """Opens a meson.build file and returns the file with literal subdir loops converted
    to single static assignment form.
    """

def inspect_files(target: Path, rel_path: Path, found_files: list[str], extra_files: list[str]) -> dict[str, list[str]]:
    ...

def process(target: Path, rel_path: Path, found_files: list[str] | None = ...) -> dict[str, list[str]]:
    """Process an OZI project build definition's files."""

def validate(target: Path, rel_path: Path, subdirs: list[str], children: set[str] | None) -> Generator[Path, None, None]:
    """Validate an OZI standard build definition's directories."""

def walk(
    target: Path,
    rel_path: Path,
    found_files: list[str] | None = None,
    project_name: str | None = None,
) -> Generator[dict[Path, dict[str, list[str]]], None, None]:
    """Walk an OZI standard build definition directory."""



"""
This type stub file was generated by pyright.
"""

from typing import Optional

class EmailNotValidError(ValueError):
    """Parent class of all exceptions raised by this module."""


class EmailSyntaxError(EmailNotValidError):
    """Exception raised when an email address fails validation because of its form."""


class EmailUndeliverableError(EmailNotValidError):
    """Exception raised when an email address fails validation because its domain name does not appear deliverable."""


class ValidatedEmail:
    """The validate_email function returns objects of this type holding the normalized form of the email address
    and other information."""
    original: str
    normalized: str
    local_part: str
    domain: str
    domain_address: object
    ascii_email: Optional[str]
    ascii_local_part: Optional[str]
    ascii_domain: str
    smtputf8: bool
    mx: list
    mx_fallback_type: str
    def __init__(self, **kwargs) -> None:
        ...


    def __getattr__(self, key): # -> str:
        ...

    @property
    def email(self): # -> str:
        ...

    def __getitem__(self, key): # -> str | bool | list[Any] | None:
        ...

    def __eq__(self, other) -> bool:
        ...

    def as_constructor(self): # -> str:
        ...

    def as_dict(self): # -> dict[str, Any]:
        ...






"""
This type stub file was generated by pyright.
"""

from collections.abc import Callable
from collections.abc import Mapping
from dataclasses import dataclass
from pathlib import Path
from typing import TYPE_CHECKING
from typing import Annotated
from typing import Union

from jinja2 import Environment
from jinja2 import Template

"""Primitives for generating meson rewrite commands."""
if TYPE_CHECKING:
    ...

@dataclass
class RewriteCommand:
    """Meson rewriter command input"""
    active: bool = ...
    type: str = ...
    target: str = ...
    operation: str = ...
    sources: list[str] = ...
    subdir: str = ...
    target_type: str = ...
    def add(self: RewriteCommand, mode: str, kind: str, source: str) -> dict[str, Union[str, list[str]]]:
        """Add sources and tests to an OZI project."""
    def rem(self: RewriteCommand, mode: str, kind: str, source: str) -> dict[str, Union[str, list[str]]]:
        """Add sources and tests to an OZI project."""

@dataclass
class Rewriter:
    """Container for Meson rewriter commands for OZI projects."""
    target: str
    name: str
    fix: str
    env: Environment
    commands: list[dict[str, str]] = ...
    path_map: Mapping[str, Callable[[str], Path]] = {}
    base_templates: dict[Annotated[str, 'fix'], Annotated[Template, 'base_template']] = {}
    def __post_init__(self: Rewriter) -> None:
        """Setup the path_map"""
    def __iadd__(self: Rewriter, other: list[str]) -> Rewriter:
        """Add a list of paths"""
    def __isub__(self: Rewriter, other: list[str]) -> Rewriter:
        """Remove a list of paths"""



# ozi/new/parser.py
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""``ozi-new`` console application.
"""
from __future__ import annotations

import argparse
import sys

from ozi_spec import METADATA
from pathvalidate.argparse import validate_filepath_arg

from ozi_core._i18n import TRANSLATION
from ozi_core.actions import CloseMatch
from ozi_core.ui.defaults import COPYRIGHT_HEAD

parser = argparse.ArgumentParser(
    prog='ozi-new',
    description=sys.modules[__name__].__doc__,
    add_help=False,
    usage='\n'.join(
        [
            f'%(prog)s [{TRANSLATION("term-options")}] | [{TRANSLATION("term-positional-args")}]',
            TRANSLATION('adm-disclaimer-text'),
        ],
    ),
)
subparser = parser.add_subparsers(help='', metavar='', dest='new')
interactive_parser = subparser.add_parser(
    'interactive',
    aliases=['i'],
    description=TRANSLATION('term-desc-new-interactive'),
    help=TRANSLATION('term-help-new-interactive'),
    prog='ozi-new interactive',
    usage=f'%(prog)s [{TRANSLATION("term-options")}] | [{TRANSLATION("term-positional-args")}]',
)
PROJECT_METADATA_HELP = f'[{TRANSLATION("term-required-metadata")}] [{TRANSLATION("term-optional-metadata")}] [{TRANSLATION("term-default-metadata")}]'  # noqa: B950,E501,RUF100
project_parser = subparser.add_parser(
    'project',
    aliases=['p'],
    description=TRANSLATION('term-desc-new-project'),
    help=TRANSLATION('term-help-new-project'),
    prog='ozi-new project',
    usage=f'%(prog)s [{TRANSLATION("term-options")}] {PROJECT_METADATA_HELP} [{TRANSLATION("term-defaults")}] target',  # noqa: B950,E501,RUF100
)
interactive_parser.add_argument(
    'target',
    type=validate_filepath_arg,
    nargs='?',
    default='.',
    help=TRANSLATION('term-help-new-target'),
)
interactive_defaults = interactive_parser.add_argument_group(TRANSLATION('term-defaults'))
interactive_defaults.add_argument(
    '-c',
    '--check-package-exists',
    default=True,
    action=argparse.BooleanOptionalAction,
    help=TRANSLATION('term-help-check-package-exists'),
)
required = project_parser.add_argument_group(TRANSLATION('term-required-metadata'))
optional = project_parser.add_argument_group(TRANSLATION('term-optional-metadata'))
defaults = project_parser.add_argument_group(TRANSLATION('term-default-metadata'))
ozi_defaults = project_parser.add_argument_group(TRANSLATION('term-defaults'))
ozi_required = project_parser.add_argument_group(TRANSLATION('term-required'))
ozi_defaults.add_argument(
    '-c',
    '--copyright-head',
    type=str,
    default=COPYRIGHT_HEAD,
    help=TRANSLATION('term-copyright-head'),
    metavar='HEADER',
)
ozi_defaults.add_argument(
    '--ci-provider',
    type=str,
    default='github',
    choices=frozenset(METADATA.spec.python.ci.providers),
    metavar='github',
    help=TRANSLATION('term-ci-provider'),
)
required.add_argument(
    '-n',
    '--name',
    type=str,
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-name'),
        text=TRANSLATION('term-help-name'),
    ),
)
required.add_argument(
    '-a',
    '--author',
    type=str,
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-author'),
        text=TRANSLATION('term-help-author'),
    ),
    action='append',
    default=[],
    metavar='AUTHOR_NAMES',
    nargs='?',
)
required.add_argument(
    '-e',
    '--author-email',
    type=str,
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-email'),
        text=TRANSLATION('term-help-author-email'),
    ),
    default=[],
    metavar='AUTHOR_EMAILS',
    nargs='?',
    action='append',
)
required.add_argument(
    '-s',
    '--summary',
    type=str,
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-summary'),
        text=TRANSLATION('term-help-summary'),
    ),
)
required.add_argument(
    '-p',
    '--home-page',
    type=str,
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-homepage'),
        text=TRANSLATION('term-help-homepage'),
    ),
)
required.add_argument(
    '--license-expression',
    type=str,
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-license-expression'),
        text=TRANSLATION('term-help-license-expression'),
    ),
)
required.add_argument(
    '-l',
    '--license',
    type=str,
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-license'),
        text=TRANSLATION('term-help-license'),
    ),
    action=CloseMatch,
)
defaults.add_argument(
    '--audience',
    '--intended-audience',
    metavar='AUDIENCE_NAMES',
    type=str,
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('edit-menu-btn-audience'),
        text=TRANSLATION('term-help-audience'),
        default=str(METADATA.spec.python.pkg.info.classifiers.intended_audience),
    ),
    default=METADATA.spec.python.pkg.info.classifiers.intended_audience,
    nargs='?',
    action='append',
)
defaults.add_argument(
    '--typing',
    type=str,
    choices=frozenset(('Typed', 'Stubs Only')),
    nargs='?',
    metavar='PY_TYPED_OR_STUBS',
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('edit-menu-btn-typing'),
        text=TRANSLATION('term-help-typing'),
        default=str(METADATA.spec.python.pkg.info.classifiers.typing),
    ),
    default=METADATA.spec.python.pkg.info.classifiers.typing,
)
defaults.add_argument(
    '--environment',
    metavar='ENVIRONMENT_NAMES',
    default=METADATA.spec.python.pkg.info.classifiers.environment,
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('edit-menu-btn-environment'),
        text=TRANSLATION('term-help-environment'),
        default=str(METADATA.spec.python.pkg.info.classifiers.environment),
    ),
    action='append',
    nargs='?',
    type=str,
)
defaults.add_argument(
    '--license-file',
    default='LICENSE.txt',
    metavar='LICENSE_FILENAME',
    choices=frozenset(('LICENSE.txt',)),
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('edit-menu-btn-license-file'),
        text=TRANSLATION('term-help-license-file'),
        default='LICENSE.txt',
    ),
    type=str,
)
optional.add_argument(
    '--keywords',
    default='',
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-keywords'),
        text=TRANSLATION('term-help-keywords'),
    ),
    type=str,
)
optional.add_argument(
    '--maintainer',
    default=[],
    action='append',
    nargs='?',
    metavar='MAINTAINER_NAMES',
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-maintainer'),
        text=TRANSLATION('term-help-maintainer'),
    ),
)
optional.add_argument(
    '--maintainer-email',
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-maintainer-email'),
        text=TRANSLATION('term-help-maintainer-email'),
    ),
    action='append',
    metavar='MAINTAINER_EMAILS',
    default=[],
    nargs='?',
)
optional.add_argument(
    '--framework',
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-framework'),
        text=TRANSLATION('term-help-framework'),
    ),
    metavar='FRAMEWORK_NAMES',
    action='append',
    type=str,
    nargs='?',
    default=[],
)
optional.add_argument(
    '--project-url',
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-project-url'),
        text=TRANSLATION('term-help-project-url'),
    ),
    action='append',
    metavar='PROJECT_URLS',
    default=[],
    nargs='?',
)
defaults.add_argument(
    '--language',
    '--natural-language',
    metavar='LANGUAGE_NAMES',
    default=['English'],
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('edit-menu-btn-language'),
        text=TRANSLATION('term-help-language'),
        default=str(['English']),
    ),
    action='append',
    type=str,
    nargs='?',
)
optional.add_argument(
    '--topic',
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-topic'),
        text=TRANSLATION('term-help-topic'),
    ),
    nargs='?',
    metavar='TOPIC_NAMES',
    action='append',
    type=str,
    default=[],
)
defaults.add_argument(
    '--status',
    '--development-status',
    action=CloseMatch,
    default=METADATA.spec.python.pkg.info.classifiers.development_status,
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('edit-menu-btn-status'),
        text=TRANSLATION('term-help-status'),
        default=str(['1 - Planning']),
    ),
    type=str,
)
defaults.add_argument(
    '--long-description-content-type',
    '--readme-type',
    metavar='README_TYPE',
    default='rst',
    choices=('rst', 'md', 'txt'),
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('edit-menu-btn-readme-type'),
        text=str(('rst', 'md', 'txt')),
        default='rst',
    ),
)
optional.add_argument(
    '-r',
    '--dist-requires',
    '--requires-dist',
    help=TRANSLATION(
        'term-help',
        name=TRANSLATION('edit-menu-btn-requires-dist'),
        text=TRANSLATION('term-help-requires-dist'),
    ),
    action='append',
    type=str,
    nargs='?',
    default=[],
    metavar='DIST_REQUIRES',
)

output = parser.add_mutually_exclusive_group()
output.add_argument('-h', '--help', action='help', help=TRANSLATION('term-help-help'))
ozi_defaults.add_argument(
    '--verify-email',
    default=False,
    action=argparse.BooleanOptionalAction,
    help=TRANSLATION('term-help-verify-email'),
)
ozi_defaults.add_argument(
    '--update-wrapfile',
    action=argparse.BooleanOptionalAction,
    default=False,
    help=TRANSLATION('term-help-update-wrapfile'),
)
ozi_defaults.add_argument(
    '--enable-cython',
    default=False,
    action=argparse.BooleanOptionalAction,
    help=TRANSLATION('term-help-enable-cython'),
)
ozi_defaults.add_argument(
    '--enable-uv',
    default=False,
    action=argparse.BooleanOptionalAction,
    help=TRANSLATION('term-help-enable-uv'),
)
ozi_defaults.add_argument(
    '--github-harden-runner',
    default=False,
    action=argparse.BooleanOptionalAction,
    help=TRANSLATION('term-help-github-harden-runner'),
)
ozi_defaults.add_argument(
    '--strict',
    default=False,
    action=argparse.BooleanOptionalAction,
    help=TRANSLATION('term-help-strict'),
)
ozi_defaults.add_argument(
    '--allow-file',
    help=TRANSLATION(
        'term-help-default',
        name=TRANSLATION('term-help-name-allow-file'),
        text=TRANSLATION('term-help-allow-file'),
        default=str(METADATA.spec.python.src.allow_files),
    ),
    action='append',
    type=str,
    nargs='?',
    metavar='ALLOW_FILE_PATTERNS',
    default=METADATA.spec.python.src.allow_files,
)
ozi_required.add_argument(
    'target',
    type=validate_filepath_arg,
    nargs='?',
    default='.',
    help=TRANSLATION('term-help-new-target'),
)



"""
This type stub file was generated by pyright.
"""

from functools import lru_cache

from prompt_toolkit.document import Document
from prompt_toolkit.validation import ThreadedValidator
from prompt_toolkit.validation import Validator

"""Validators for interactive prompts."""

@lru_cache
def pypi_package_exists(package: str) -> bool:
    """Check if a package name exists on PyPI."""

class ProjectNameValidator(Validator):
    """Validate that a package name is valid."""

    def validate(self, document: Document) -> None: ...

class NotReservedValidator(ThreadedValidator):
    """Validate that a package name is available."""

    def validate(self, document: Document) -> None: ...

class LengthValidator(Validator):
    """Validate text is between 1 and 512 chartacters in length."""

    def validate(self, document: Document) -> None: ...

class PackageValidator(Validator):
    """Validate a package name exists on PyPI."""

    def validate(self, document: Document) -> None: ...

def validate_message(text: str, validator: Validator) -> tuple[bool, str]:
    """Validate a string.

    :param text: string to validate
    :type text: str
    :param validator: validator instance
    :type validator: Validator
    :return: validation, error message
    :rtype: tuple[bool, str]
    """



import logging
import logging.handlers
from pathlib import Path

LOG_PATH: Path

class PytestFilter(logging.Filter):
    def filter(self: PytestFilter, record: logging.LogRecord) -> bool: ...

class CompressedRotatingFileHandler(logging.handlers.RotatingFileHandler):
    def doRollover(self) -> None: ...

def config_logger() -> None: ...



"""
This type stub file was generated by pyright.
"""

import sys
from email.message import Message
from pathlib import Path
from typing import TYPE_CHECKING

"""Find missing OZI project files."""
if sys.version_info >= (3, 11):
    ...
else:
    ...
if TYPE_CHECKING:
    ...
PKG_INFO = ...
readme_ext_to_content_type = ...

def get_relpath_expected_files(
    kind: str | None,
    name: str,
) -> tuple[Path, tuple[str, ...] | tuple[()]]: ...

def render_requirements(target: Path) -> str:
    """Render requirements.in as it would appear in PKG-INFO"""

def render_pkg_info(target: Path, name: str, _license: str) -> Message:
    """Render PKG-INFO as it would be produced during packaging."""

def python_support(pkg_info: Message) -> set[tuple[str, str]]:
    """Check PKG-INFO Message for python support."""

def required_extra_pkg_info(pkg_info: Message) -> dict[str, str]:
    """Check missing required OZI extra PKG-INFO"""

def required_pkg_info(target: Path) -> tuple[str, dict[str, str]]:
    """Find missing required PKG-INFO"""

def required_files(kind: str, target: Path, name: str) -> list[str]:
    """Count missing files required by OZI"""

def report(target: Path) -> tuple[str, Message | None, list[str], list[str], list[str]]:
    """Report missing OZI project files
    :param target: Relative path to target directory.
    :return: Normalized Name, PKG-INFO, found_root, found_sources, found_tests
    """




"""
This type stub file was generated by pyright.
"""

from argparse import Namespace
from typing import Any
from typing import Sequence

from pyparsing import ParseResults

from ozi_core.vendor.email_validator import ValidatedEmail

_CLASSIFIERS: set[str] = ...

def valid_classifier(classifier: str) -> None:
    """Validate a classifier string"""

def valid_project_url(project_url: Sequence[str]) -> None:
    """Validate a list of project urls strings of the format ``name,url``."""

def valid_home_page(home_page: str) -> None:
    """Validate a project homepage url"""

def valid_summary(summary: str) -> None:
    """Validate project summary length."""

def valid_contact_info(author: str, maintainer: str, author_email: Sequence[str], maintainer_email: Sequence[str]) -> None:
    """Validate project contact info metadata.

    :param author: comma-separated author names
    :type author: str
    :param maintainer: comma-separated maintainer names
    :type maintainer: str
    :param author_email: author email addresses
    :type author_email: Sequence[str]
    :param maintainer_email: maintainer email addresses
    :type maintainer_email: Sequence[str]
    """

def valid_license(_license: str, license_expression: str) -> str:
    """Validate license and check against license expression."""

def valid_copyright_head(copyright_head: str, project_name: str, license_file: str) -> str:
    """Validate a copyright header.

    :param copyright_head: the header text in full
    :type copyright_head: str
    :param project_name: the project name
    :type project_name: str
    :param license_file: the license filename
    :type license_file: str
    """

def valid_project_name(name: str | ParseResults) -> None:
    """Validate a project name."""

def valid_spdx(expr: Any | ParseResults) -> None:
    """Validate a SPDX license expression."""

def valid_email(email: str, verify: bool = ...) -> ValidatedEmail | None:
    """Validate a single email address."""

def valid_emails(author_email: list[str], maintainer_email: list[str], verify: bool) -> tuple[list[str], list[str]]:
    """Validate lists of author and maintainer emails."""

def preprocess_arguments(project: Namespace) -> Namespace:
    """Preprocess (validate) arguments for project namespace."""

def postprocess_arguments(project: Namespace) -> Namespace:
    """Postprocess (normalize) arguments for project namespace."""




"""
This type stub file was generated by pyright.
"""

from __future__ import annotations

import sys
from argparse import Namespace

"""ozi-new: quick-start OZI project creation script."""
if sys.platform != 'win32':
    ...  # import curses
else:
    ...

def project(project: Namespace) -> None:
    """Create a new project in a target directory."""

def wrap(project: Namespace) -> None:
    """Create a new wrap file for publishing. Not a public function."""

def main(args: list[str] | None = ...) -> None:
    """Main ozi.new entrypoint."""




# ozi/new/validate.py
# Part of the OZI Project, under the Apache License v2.0 with LLVM Exceptions.
# See LICENSE.txt for license information.
# SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
"""ozi-new input validation."""
from __future__ import annotations

import re
from pathlib import Path
from typing import TYPE_CHECKING
from typing import Any
from typing import Sequence
from urllib.parse import urlparse

from ozi_spec import METADATA
from pyparsing import Combine
from pyparsing import ParseException
from pyparsing import ParseResults
from pyparsing import Regex
from tap_producer import TAP
from trove_classifiers import classifiers

from ozi_core._i18n import TRANSLATION
from ozi_core.spdx import spdx_license_expression
from ozi_core.ui.defaults import COPYRIGHT_HEAD
from ozi_core.vendor.email_validator import EmailNotValidError
from ozi_core.vendor.email_validator import EmailSyntaxError
from ozi_core.vendor.email_validator import ValidatedEmail
from ozi_core.vendor.email_validator import validate_email

if TYPE_CHECKING:  # pragma: no cover
    from argparse import Namespace

_CLASSIFIERS = {i.partition(' :: ')[2].strip() for i in classifiers}


def valid_classifier(classifier: str) -> None:
    """Validate a classifier string"""
    if classifier in _CLASSIFIERS or classifier in classifiers:
        TAP.ok(TRANSLATION('term-classifier'), classifier)
    else:  # pragma: no cover
        TAP.not_ok(TRANSLATION('term-classifier'), classifier)


def valid_project_url(project_url: Sequence[str]) -> None:
    """Validate a list of project urls strings of the format ``name,url``."""
    for name, url in [str(i).split(',') for i in project_url]:
        if len(name) > 32:
            TAP.not_ok(
                TRANSLATION('edit-menu-btn-project-url'),
                TRANSLATION('term-tap-name-gt32'),
            )
        parsed_url = urlparse(url)
        match parsed_url:
            case p if p.scheme != 'https':
                TAP.comment(TRANSLATION('term-tap-https-only'))
                TAP.not_ok(
                    TRANSLATION('edit-menu-btn-project-url'),
                    TRANSLATION('term-tap-unsupported-url-scheme'),
                )
            case p if p.netloc == '':
                TAP.not_ok(
                    TRANSLATION('edit-menu-btn-project-url'),
                    TRANSLATION('term-tap-empty-netloc'),
                )
            case _:
                TAP.ok(
                    TRANSLATION('edit-menu-btn-project-url'),
                    TRANSLATION('term-tap-netloc'),
                )


def valid_home_page(home_page: str) -> None:
    """Validate a project homepage url"""
    home_url = urlparse(home_page)
    if home_url.scheme != 'https':  # pragma: defer to good-issue
        TAP.comment(TRANSLATION('term-tap-https-only'))
        TAP.not_ok(
            TRANSLATION('edit-menu-btn-homepage'),
            TRANSLATION('term-tap-unsupported-url-scheme'),
        )
    else:
        TAP.ok(TRANSLATION('edit-menu-btn-homepage'), TRANSLATION('term-tap-url-scheme'))
    if home_url.netloc == '':  # pragma: defer to good-issue
        TAP.not_ok(
            TRANSLATION('edit-menu-btn-homepage'),
            TRANSLATION('term-tap-empty-netloc'),
        )
    else:
        TAP.ok(TRANSLATION('edit-menu-btn-homepage'), TRANSLATION('term-tap-netloc'))


def valid_summary(summary: str) -> None:
    """Validate project summary length."""
    if len(summary) > 512:
        TAP.not_ok(
            TRANSLATION('edit-menu-btn-summary'),
            TRANSLATION('term-tap-summary-gt512'),
        )
    else:
        TAP.ok(TRANSLATION('edit-menu-btn-summary'))


def valid_contact_info(  # noqa: C901
    author: str,
    maintainer: str,
    author_email: Sequence[str],
    maintainer_email: Sequence[str],
) -> None:
    """Validate project contact info metadata.

    :param author: comma-separated author names
    :type author: str
    :param maintainer: comma-separated maintainer names
    :type maintainer: str
    :param author_email: author email addresses
    :type author_email: Sequence[str]
    :param maintainer_email: maintainer email addresses
    :type maintainer_email: Sequence[str]
    """
    author_and_maintainer_email = False
    if set(author_email).intersection(maintainer_email):
        TAP.not_ok(
            TRANSLATION('edit-menu-btn-maintainer-email'),
            TRANSLATION('term-tap-identical-email'),
            TRANSLATION(
                'term-tap-leave-blank',
                key=TRANSLATION('edit-menu-btn-maintainer-email'),
            ),
        )
    elif any(map(len, maintainer_email)) and not any(map(len, author_email)):
        TAP.not_ok(
            TRANSLATION('edit-menu-btn-maintainer-email'),
            TRANSLATION('term-tap-leave-blank', key=TRANSLATION('edit-menu-btn-email')),
        )
    elif any(map(len, maintainer_email)) and any(map(len, author_email)):
        author_and_maintainer_email = True
        TAP.ok(TRANSLATION('edit-menu-btn-email'))
        TAP.ok(TRANSLATION('edit-menu-btn-maintainer-email'))
    else:
        TAP.ok(TRANSLATION('edit-menu-btn-email'))

    if set(author_email).intersection(maintainer_email):
        TAP.not_ok(  # pragma: defer to good-issue
            TRANSLATION('edit-menu-btn-maintainer'),
            TRANSLATION('term-tap-identical-author'),
            TRANSLATION('term-tap-leave-blank', key=TRANSLATION('edit-menu-btn-maintainer')),
        )
    elif len(maintainer) and not author:
        TAP.not_ok(
            TRANSLATION('edit-menu-btn-maintainer'),
            TRANSLATION('term-tap-not-set', key=TRANSLATION('edit-menu-btn-author')),
        )
    elif len(maintainer) and len(author):
        TAP.ok(TRANSLATION('edit-menu-btn-author'))
        TAP.ok(TRANSLATION('edit-menu-btn-maintainer'))
    elif author_and_maintainer_email and not maintainer:
        TAP.not_ok(  # pragma: defer to good issue
            TRANSLATION('edit-menu-btn-maintainer-email'),
            TRANSLATION('term-tap-not-set', key=TRANSLATION('edit-menu-btn-maintainer')),
        )
    else:
        TAP.ok(TRANSLATION('edit-menu-btn-author'))


def valid_license(_license: str, license_expression: str) -> str:
    """Validate license and check against license expression."""
    if isinstance(_license, list):  # pragma: no cover
        TAP.ok(TRANSLATION('term-tap-first-license'), skip=False, licenses=_license)
        _license = _license[0]
    possible_spdx: Sequence[str] = METADATA.spec.python.pkg.license.ambiguous.get(
        _license,
        (),
    )
    if (
        _license in iter(METADATA.spec.python.pkg.license.ambiguous)
        and len(METADATA.spec.python.pkg.license.ambiguous[_license]) > 1
        and license_expression.split(' ')[0] not in possible_spdx
    ):  # pragma: no cover
        TAP.not_ok(
            TRANSLATION('edit-menu-btn-license'),
            TRANSLATION('term-tap-ambiguous-pep639'),
            _license,
            skip=False,
            message=TRANSLATION('term-tap-ambiguous-license'),
            licenses=tuple(possible_spdx),
            reference='https://github.com/pypa/trove-classifiers/issues/17',
        )
    else:
        TAP.ok(TRANSLATION('edit-menu-btn-license'))
    return _license


def valid_copyright_head(copyright_head: str, project_name: str, license_file: str) -> str:
    """Validate a copyright header.

    :param copyright_head: the header text in full
    :type copyright_head: str
    :param project_name: the project name
    :type project_name: str
    :param license_file: the license filename
    :type license_file: str
    """
    if copyright_head == COPYRIGHT_HEAD:  # pragma: no cover
        copyright_head = copyright_head.format(
            project_name=project_name,
            license_file=license_file,
        )
        TAP.ok(TRANSLATION('term-copyright-head'), TRANSLATION('term-defaults'))
    else:
        if project_name not in copyright_head:  # pragma: no cover
            TAP.comment(
                TRANSLATION('term-copyright-head'),
                TRANSLATION('term-tap-header-name-not-found'),
            )
        if license_file not in copyright_head:  # pragma: no cover
            TAP.comment(
                TRANSLATION('term-copyright-head'),
                TRANSLATION('term-tap-header-license-file-not-found'),
            )
        TAP.ok(TRANSLATION('term-copyright-head'), TRANSLATION('term-custom'))
    return copyright_head


def valid_project_name(name: str | ParseResults) -> None:
    """Validate a project name."""
    try:
        Regex('^([A-Z]|[A-Z][A-Z0-9._-]*[A-Z0-9])$', re.IGNORECASE).set_name(
            TRANSLATION('edit-menu-btn-name'),
        ).parse_string(str(name))
        TAP.ok(TRANSLATION('edit-menu-btn-name'))
    except ParseException as e:
        TAP.not_ok(*str(e).split('\n'))


def valid_spdx(expr: Any | ParseResults) -> None:
    """Validate a SPDX license expression."""
    try:
        expr = Combine(
            spdx_license_expression,
            join_string=' ',
        ).parse_string(
            str(expr),
        )[0]
        TAP.ok(TRANSLATION('edit-menu-btn-license-expression'))
    except ParseException as e:  # pragma: defer to good-issue
        TAP.not_ok(TRANSLATION('edit-menu-btn-license-expression'), *str(e).split('\n'))


def valid_email(email: str, verify: bool = False) -> ValidatedEmail | None:
    """Validate a single email address."""
    try:
        return validate_email(email, check_deliverability=verify)
    except (EmailNotValidError, EmailSyntaxError) as e:
        TAP.not_ok(*str(e).split('\n'))
        return None  # pragma: no cover


def valid_emails(
    author_email: list[str],
    maintainer_email: list[str],
    verify: bool,
) -> tuple[list[str], list[str]]:
    """Validate lists of author and maintainer emails."""
    _author_email = []
    _maintainer_email = []
    for email in set(author_email).union(maintainer_email):
        emailinfo = valid_email(email, verify=verify)
        match emailinfo:
            case ValidatedEmail() if email in author_email:
                _author_email += [emailinfo.normalized]
                TAP.ok(TRANSLATION('edit-menu-btn-email'))
            case ValidatedEmail() if email in maintainer_email:
                _maintainer_email += [emailinfo.normalized]
                TAP.ok(TRANSLATION('edit-menu-btn-maintainer-email'))
            case None:  # pragma: no cover
                continue
    return _author_email, _maintainer_email


def _valid_project(project: Namespace) -> Namespace:
    """Validate a project namespace."""
    valid_project_name(name=project.name)
    valid_summary(project.summary)
    project.license = valid_license(
        _license=project.license,
        license_expression=project.license_expression,
    )
    valid_home_page(home_page=project.home_page)
    valid_project_url(project_url=project.project_url)
    project.copyright_head = valid_copyright_head(
        copyright_head=project.copyright_head,
        project_name=project.name,
        license_file=project.license_file,
    )
    valid_spdx(project.license_expression)
    valid_contact_info(
        author=project.author,
        maintainer=project.maintainer,
        author_email=project.author_email,
        maintainer_email=project.maintainer_email,
    )
    for i in [
        project.audience,
        project.environment,
        project.framework,
        project.topic,
    ]:
        for classifier in i:
            valid_classifier(classifier)
    return project


def preprocess_arguments(project: Namespace) -> Namespace:
    """Preprocess (validate) arguments for project namespace."""
    if project.strict:
        with TAP.strict():  # pragma: no cover  # pyright: ignore
            return _valid_project(project)
    else:
        return _valid_project(project)


def postprocess_arguments(project: Namespace) -> Namespace:
    """Postprocess (normalize) arguments for project namespace."""
    project.author_email, project.maintainer_email = valid_emails(
        author_email=project.author_email,
        maintainer_email=project.maintainer_email,
        verify=project.verify_email,
    )
    project.keywords = project.keywords.split(',')
    project.name = re.sub(r'[-_.]+', '-', project.name)
    project.target = Path(project.target)
    project.topic = list(set(project.topic))
    project.dist_requires = list(set(project.dist_requires))
    if any(
        i for i in project.target.iterdir() if i not in project.allow_file
    ):  # defer to good-issue
        TAP.not_ok('target', TRANSLATION('term-tap-target-not-empty'), skip=True)
    match project.ci_provider:
        case 'github':
            pass
        case _:  # pragma: no cover
            TAP.not_ok(
                TRANSLATION('term-tap-invalid-ci-provider', ciprovider=project.ci_provider),
            )
    project.allow_file = set(map(Path, project.allow_file))
    return project



"""
This type stub file was generated by pyright.
"""

from __future__ import annotations

from typing import TYPE_CHECKING
from typing import NoReturn

"""ozi-fix: Project fix script that outputs a meson rewriter JSON array."""
if TYPE_CHECKING:
    ...
def main() -> NoReturn:
    """Main ozi.fix entrypoint."""




"""
``ozi-new`` interactive prompts
"""

from __future__ import annotations

import os
import sys
from dataclasses import asdict
from typing import TYPE_CHECKING
from unittest.mock import Mock

if sys.platform != 'win32':
    import curses
else:
    curses = Mock()
    curses.tigetstr = lambda x: b''
    curses.setupterm = lambda: None

from ozi_core.config import read_user_config
from ozi_core.new.interactive.project import Project

if TYPE_CHECKING:
    from argparse import Namespace


def interactive_prompt(project: Namespace) -> list[str]:  # pragma: no cover
    curses.setupterm()
    e3 = curses.tigetstr('E3') or b''
    clear_screen_seq = curses.tigetstr('clear') or b''
    os.write(sys.stdout.fileno(), e3 + clear_screen_seq)
    config = asdict(read_user_config())
    project_prompt = Project(
        allow_file=config['new']['allow_file'],
        check_package_exists=project.check_package_exists,
        ci_provider=config['new']['ci_provider'],
        copyright_head=config['new']['copyright_head'],
        enable_cython=config['new']['enable_cython'],
        enable_uv=config['new']['enable_uv'],
        github_harden_runner=config['new']['github_harden_runner'],
        strict=config['new']['strict'],
        verify_email=config['new']['verify_email'],
    )
    return project_prompt()



"""
This type stub file was generated by pyright.
"""

"""Core OZI packaging management plane module.

.. versionremoved:: 1.2
   The module ``filters`` was moved to ``blastpipe.ozi_templates.filters``

.. versionchanged:: 1.12

   The ozi_templates module was moved out of blastpipe.
"""

def current_version() -> str:
    """Returns the currently installed version of OZI.

    .. versionchanged:: 1.14

      Previously was part of the ozi.spec submodule.

    """

__version__: str
__author__: str
__all__ = ('__version__', '__author__', '__doc__')
